@@ -1,21 +1,42 @@
 /*
- * TextBuffer.java
+ * java-gnome, a UI library for writing GTK and GNOME programs from Java!
  *
- * Copyright (c) 2007-2008 Operational Dynamics Consulting Pty Ltd, and Others
+ * Copyright © 2007-2010 Operational Dynamics Consulting, Pty Ltd and Others
  *
- * The code in this file, and the library it is a part of, are made available
- * to you by the authors under the terms of the "GNU General Public Licence,
- * version 2" plus the "Classpath Exception" (you may link to this code as a
- * library into other programs provided you don't make a derivation of it).
- * See the LICENCE file for the terms governing usage and redistribution.
+ * The code in this file, and the program it is a part of, is made available
+ * to you by its authors as open source software: you can redistribute it
+ * and/or modify it under the terms of the GNU General Public License version
+ * 2 ("GPL") as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE. See the GPL for more details.
+ *
+ * You should have received a copy of the GPL along with this program. If not,
+ * see http://www.gnu.org/licenses/. The authors of this program may be
+ * contacted through http://java-gnome.sourceforge.net/.
+ *
+ * Linking this library statically or dynamically with other modules is making
+ * a combined work based on this library. Thus, the terms and conditions of
+ * the GPL cover the whole combination. As a special exception (the
+ * "Claspath Exception"), the copyright holders of this library give you
+ * permission to link this library with independent modules to produce an
+ * executable, regardless of the license terms of these independent modules,
+ * and to copy and distribute the resulting executable under terms of your
+ * choice, provided that you also meet, for each linked independent module,
+ * the terms and conditions of the license of that module. An independent
+ * module is a module which is not derived from or based on this library. If
+ * you modify this library, you may extend the Classpath Exception to your
+ * version of the library, but you are not obligated to do so. If you do not
+ * wish to do so, delete this exception statement from your version.
  */
 package org.gnome.gtk;
 
+import java.util.Collection;
+
 import org.gnome.gdk.Pixbuf;
 import org.gnome.glib.Object;
 
-import static org.gnome.gtk.TextTagTable.getDefaultTable;
-
 /**
  * A TextBuffer is a powerful mechanism for storing and manipulating text. It
  * exists primarily to be the model that backs the view of one or more
@@ -146,11 +167,32 @@ public class TextBuffer extends Object
      */
     public static final char OBJECT_REPLACEMENT_CHARACTER = 0xFFFC;
 
+    private static TextTagTable defaultTable;
+
+    /**
+     * We maintain a single default TextTagTable to facilitate no-arg
+     * convenience constructors for TextTag and TextBuffer. Get (and
+     * initialize if necessary) this table.
+     */
+    /*
+     * synchronized?
+     */
+    protected static TextTagTable getDefaultTable() {
+        if (defaultTable == null) {
+            defaultTable = new TextTagTable();
+        }
+        return defaultTable;
+    }
+
     private final boolean usingDefaultTable;
 
     protected TextBuffer(long pointer) {
+        this(pointer, false);
+    }
+
+    protected TextBuffer(long pointer, boolean usingDefaultTable) {
         super(pointer);
-        usingDefaultTable = false;
+        this.usingDefaultTable = usingDefaultTable;
     }
 
     /**
@@ -167,8 +209,7 @@ public class TextBuffer extends Object
      * @since 4.0.9
      */
     public TextBuffer() {
-        super(GtkTextBuffer.createTextBuffer(getDefaultTable()));
-        usingDefaultTable = true;
+        this(GtkTextBuffer.createTextBuffer(getDefaultTable()), true);
     }
 
     /**
@@ -178,8 +219,7 @@ public class TextBuffer extends Object
      * @since 4.0.9
      */
     public TextBuffer(TextTagTable tags) {
-        super(GtkTextBuffer.createTextBuffer(tags));
-        usingDefaultTable = false;
+        this(GtkTextBuffer.createTextBuffer(tags), false);
     }
 
     /**
@@ -192,8 +232,8 @@ public class TextBuffer extends Object
     }
 
     /**
-     * Returns the text in the range <code>start</code> .. <code>end</code>.
-     * Excludes undisplayed text (text marked with tags that set the
+     * Returns the text in the range <code>start</code> .. <code>finish</code>
+     * . Excludes undisplayed text (text marked with tags that set the
      * <var>invisibility</var> attribute) if <code>includeHidden</code> is
      * <code>false</code>. Does not include characters representing embedded
      * images, so indexes into the returned string do not correspond to
@@ -201,8 +241,23 @@ public class TextBuffer extends Object
      * 
      * @since 4.0.9
      */
-    public String getText(TextIter start, TextIter end, boolean includeHidden) {
-        return GtkTextBuffer.getText(this, start, end, includeHidden);
+    public String getText(TextIter start, TextIter finish, boolean includeHidden) {
+        return GtkTextBuffer.getText(this, start, finish, includeHidden);
+    }
+
+    /**
+     * Returns the text in the range <code>start</code> .. <code>finish</code>
+     * with the {@link #OBJECT_REPLACEMENT_CHARACTER} marking any positions
+     * that are actually anchors for Widgets or Images.
+     * 
+     * <p>
+     * Compare to {@link #getText(TextIter, TextIter, boolean) getText()},
+     * which skips over those meta positions.
+     * 
+     * @since 4.0.17
+     */
+    public String getSlice(TextIter start, TextIter finish, boolean includeHidden) {
+        return GtkTextBuffer.getSlice(this, start, finish, includeHidden);
     }
 
     /**
@@ -326,6 +381,79 @@ public class TextBuffer extends Object
         GtkTextBuffer.insertWithTags(this, position, text, -1, tag);
     }
 
+    /**
+     * Insert text at the given position, applying all of the tags specified.
+     * 
+     * @since 4.0.10
+     */
+    /*
+     * This essentially duplicates the logic in GTK's
+     * gtk_text_buffer_insert_with_tags().
+     */
+    public void insert(TextIter position, String text, TextTag[] tags) {
+        TextIter start;
+        int original;
+
+        original = position.getOffset();
+
+        GtkTextBuffer.insert(this, position, text, -1);
+
+        if (tags == null) {
+            return;
+        }
+
+        start = position.copy();
+        start.setOffset(original);
+
+        for (TextTag tag : tags) {
+            if (tag == null) {
+                continue;
+            }
+            checkTag(tag);
+            GtkTextBuffer.applyTag(this, tag, start, position);
+        }
+    }
+
+    /**
+     * Insert text at the given position, applying all of the tags specified.
+     * 
+     * <p>
+     * <i> Having an overload taking a generic is somewhat unusual in
+     * java-gnome, but people maintain a (fairly rapidly changing) List or Set
+     * with the TextTags they are currently inserting so frequently that we
+     * wanted to support this use case efficiently.</i>
+     * 
+     * @since 4.0.10
+     */
+    /*
+     * Not to mention that Collection to array conversion is one of the uglier
+     * idioms in Java. Yes, this is a copy of the code in other insert()
+     * method above; we want to save gratuitous unnecessary array creation.
+     */
+    public void insert(TextIter position, String text, Collection<TextTag> tags) {
+        final TextIter start;
+        final int original;
+
+        original = position.getOffset();
+
+        GtkTextBuffer.insert(this, position, text, -1);
+
+        if (tags == null) {
+            return;
+        }
+
+        start = position.copy();
+        start.setOffset(original);
+
+        for (TextTag tag : tags) {
+            if (tag == null) {
+                continue;
+            }
+            checkTag(tag);
+            GtkTextBuffer.applyTag(this, tag, start, position);
+        }
+    }
+
     /**
      * Insert the text at the current cursor position.
      * 
@@ -339,7 +467,13 @@ public class TextBuffer extends Object
      * Like {@link #insert(TextIter, String) insert()}, but the insertion will
      * not occur if <code>pos</code> points to a non-editable location in the
      * buffer - meaning that it is enclosed in TextTags that mark the area
-     * non-editable.<br/>
+     * non-editable.
+     * 
+     * <p>
+     * This method is conducted as a user action, so the
+     * <code>TextBuffer.BeginUserAction</code> and
+     * <code>TextBuffer.EndUserAction</code> signals will be raised before and
+     * after carrying out the insertion, respectively.
      * 
      * @param pos
      *            Position to insert at.
@@ -358,6 +492,27 @@ public class TextBuffer extends Object
         GtkTextBuffer.insertInteractive(this, pos, text, -1, defaultEditability);
     }
 
+    /**
+     * Delete text like {@link #delete(TextIter, TextIter) delete()}, but only
+     * if the range given is editable.
+     * 
+     * <p>
+     * See {@link #insertInteractive(TextIter, String, boolean)
+     * insertInteractive()} for a discussion of the
+     * <code>defaultEditability</code> parameter.
+     * 
+     * <p>
+     * This method is conducted as a user action, so the
+     * <code>TextBuffer.BeginUserAction</code> and
+     * <code>TextBuffer.EndUserAction</code> signals will be raised before and
+     * after carrying out the deletion.
+     * 
+     * @since 4.0.13
+     */
+    public void deleteInteractive(TextIter start, TextIter finish, boolean defaultEditability) {
+        GtkTextBuffer.deleteInteractive(this, start, finish, defaultEditability);
+    }
+
     /**
      * Inserts an image at the cursor position.
      * 
@@ -504,13 +659,35 @@ public class TextBuffer extends Object
 
     /**
      * Apply the selected tag on the area in the TextBuffer between the start
-     * and end positions.
+     * and finish positions.
      * 
      * @since 4.0.9
      */
-    public void applyTag(TextTag tag, TextIter start, TextIter end) {
+    public void applyTag(TextTag tag, TextIter start, TextIter finish) {
         checkTag(tag);
-        GtkTextBuffer.applyTag(this, tag, start, end);
+        GtkTextBuffer.applyTag(this, tag, start, finish);
+    }
+
+    /**
+     * Apply an array of TextTags to the given range.
+     * 
+     * @since 4.0.10
+     */
+    /*
+     * Convenience method. This doesn't need to be here, but it lends a
+     * certain elegance when used alongside the insert() overload
+     */
+    public void applyTag(TextTag[] tags, TextIter start, TextIter finish) {
+        if (tags == null) {
+            return;
+        }
+        for (TextTag tag : tags) {
+            if (tag == null) {
+                continue;
+            }
+            checkTag(tag);
+            GtkTextBuffer.applyTag(this, tag, start, finish);
+        }
     }
 
     /**
@@ -531,12 +708,12 @@ public class TextBuffer extends Object
      * 
      * <p>
      * <i>The native GTK function has these arguments reversed but start and
-     * end make more sense in consecutive order.</i>
+     * finish make more sense in consecutive order.</i>
      * 
      * @since 4.0.9
      */
-    public void selectRange(TextIter start, TextIter end) {
-        GtkTextBuffer.selectRange(this, end, start);
+    public void selectRange(TextIter start, TextIter finish) {
+        GtkTextBuffer.selectRange(this, finish, start);
     }
 
     /**
@@ -546,8 +723,23 @@ public class TextBuffer extends Object
      * 
      * @since 4.0.9
      */
-    public void removeTag(TextTag tag, TextIter start, TextIter end) {
-        GtkTextBuffer.removeTag(this, tag, start, end);
+    public void removeTag(TextTag tag, TextIter start, TextIter finish) {
+        GtkTextBuffer.removeTag(this, tag, start, finish);
+    }
+
+    /**
+     * Remove all TextTags that may be present in a range.
+     * 
+     * <p>
+     * Beware that <b>all</b> tags between <code>start</code> and
+     * <code>end</code> will be nuked, not just ones set by the piece of code
+     * you happen to be working in. That should be obvious, but apparently
+     * people often make mistakes with this.
+     * 
+     * @since 4.0.10
+     */
+    public void removeAllTags(TextIter start, TextIter finish) {
+        GtkTextBuffer.removeAllTags(this, start, finish);
     }
 
     /**
@@ -694,7 +886,7 @@ public class TextBuffer extends Object
      * calling:
      * 
      * <pre>
-     * deleted = buffer.getText(start, end, false);
+     * deleted = buffer.getText(start, finish, false);
      * </pre>
      * 
      * <p>
@@ -709,12 +901,12 @@ public class TextBuffer extends Object
      */
     /*
      * TODO Can anyone explain how to stop a deletion from occurring in
-     * response to the value of the text between start and end? The default
+     * response to the value of the text between start and finish? The default
      * handler will nuke the text.
      */
     public interface DeleteRange extends GtkTextBuffer.DeleteRangeSignal
     {
-        public void onDeleteRange(TextBuffer source, TextIter start, TextIter end);
+        public void onDeleteRange(TextBuffer source, TextIter start, TextIter finish);
     }
 
     /**
@@ -728,9 +920,9 @@ public class TextBuffer extends Object
     }
 
     /**
-     * Delete text between <code>start</code> and <code>end</code>. (The order
-     * of the two TextIters doesn't matter; this method will delete between
-     * the two regardless).
+     * Delete text between <code>start</code> and <code>finish</code>. (The
+     * order of the two TextIters doesn't matter; this method will delete
+     * between the two regardless).
      * 
      * <p>
      * The two TextIters passed to <code>delete()</code> will be reset so as
@@ -740,7 +932,303 @@ public class TextBuffer extends Object
      * 
      * @since 4.0.9
      */
-    public void delete(TextIter start, TextIter end) {
-        GtkTextBuffer.delete(this, start, end);
+    public void delete(TextIter start, TextIter finish) {
+        GtkTextBuffer.delete(this, start, finish);
+    }
+
+    /**
+     * Manually move a TextMark to a new location.
+     * 
+     * <p>
+     * Note that if you're trying to move the "cursor", then you are much
+     * better off calling {@link #placeCursor(TextIter) placeCursor()} as that
+     * simultaneously moves both the <var>selection-bound</var> TextMark as
+     * well as the <var>insert</var> one.
+     * 
+     * @since 4.0.10
+     */
+    public void moveMark(TextMark mark, TextIter where) {
+        GtkTextBuffer.moveMark(this, mark, where);
+    }
+
+    /**
+     * Signal emitted when a TextMark is set (or moved) in this TextBuffer.
+     * 
+     * <p>
+     * This can be used as a way to react to the cursor moving. The cursor is,
+     * of course, represented by the <var>insert</var> TextMark, and so,
+     * doing:
+     * 
+     * <pre>
+     * insert = buffer.getInsert();
+     * 
+     * buffer.connect(new TextBuffer.MarkSet() {
+     *     public void onMarkSet(TextBuffer source, org.gnome.gtk.TextIter location, TextMark mark) {
+     *         if (mark == insert) {
+     *             // react!
+     *         }
+     *     }
+     * });
+     * </pre>
+     * 
+     * will allow you to react to the cursor moving.
+     * 
+     * <p>
+     * Somewhat counter-intuitively, however, inserting text does <i>not</i>
+     * "move" a TextMark; the <var>insert</var> TextMark will flow right
+     * according to its gravity as text is added. Using the arrow keys or
+     * mouse to move the cursor will, on the other hand, result in this signal
+     * being emitted.
+     * 
+     * <p>
+     * While an interesting example, doing this to track the cursor changing
+     * isn't actually a good idea. You get a <code>TextBuffer.MarkSet</code>
+     * callback for <i>every</i> TextMark being set (including ones used
+     * internally by GTK!) which means lots of activity that has nothing to do
+     * with you. Use {@link TextBuffer.NotifyCursorPosition} instead.
+     * 
+     * <p>
+     * <i>Using signal is very inefficient; why this is a signal on TextBuffer
+     * and not a signal coming (only) from individual TextMarks is
+     * mystery.</i>
+     * 
+     * @author Andrew Cowie
+     * @since 4.0.10
+     */
+    public interface MarkSet extends GtkTextBuffer.MarkSetSignal
+    {
+        public void onMarkSet(TextBuffer source, TextIter location, TextMark mark);
+    }
+
+    /**
+     * Hook up a handler for <code>TextBuffer.MarkSet</code> signals on this
+     * TextBuffer.
+     * 
+     * @since 4.0.10
+     */
+    public void connect(TextBuffer.MarkSet handler) {
+        GtkTextBuffer.connect(this, handler, false);
+    }
+
+    /**
+     * The signal emitted when a TextTag is added to a range of text.
+     * 
+     * <p>
+     * If you're using this then you'll probably also need the complement of
+     * this signal, which is <code>TextBuffer.RemoveTag</code>.
+     * 
+     * <p>
+     * <i>The code to actually carry out the application of the tag is in the
+     * default handler, which will run after you return from yours.</i>
+     * 
+     * @author Andrew Cowie
+     * @since 4.0.10
+     */
+    /*
+     * TODO Which raises some questions: do we need to expose this as a
+     * connectAfter()? Also, if a tag is already present does this signal get
+     * raised anyway? Finally, do we need emitApplyTag()?
+     */
+    public interface ApplyTag extends GtkTextBuffer.ApplyTagSignal
+    {
+        void onApplyTag(TextBuffer source, TextTag tag, TextIter start, TextIter finish);
+    }
+
+    /**
+     * Hook up a handler for <code>TextBuffer.ApplyTag</code> signals on this
+     * TextBuffer.
+     * 
+     * @since 4.0.10
+     */
+    public void connect(TextBuffer.ApplyTag handler) {
+        GtkTextBuffer.connect(this, handler, false);
+    }
+
+    /**
+     * The signal emitted when a TextTag is removed from a range of text.
+     * 
+     * @author Andrew Cowie
+     * @since 4.0.10
+     */
+    public interface RemoveTag extends GtkTextBuffer.RemoveTagSignal
+    {
+        void onRemoveTag(TextBuffer source, TextTag tag, TextIter start, TextIter finish);
+    }
+
+    /**
+     * Hook up a handler for <code>TextBuffer.RemoveTag</code> signals on this
+     * TextBuffer.
+     * 
+     * @since 4.0.10
+     */
+    public void connect(TextBuffer.RemoveTag handler) {
+        GtkTextBuffer.connect(this, handler, false);
+    }
+
+    /**
+     * Signal emitted when the <var>cursor-position</var> property changes.
+     * This is a good way to find out that the cursor has moved.
+     * 
+     * 
+     * <pre>
+     * buffer.connect(new TextBuffer.NotifyCursorPosition()) {
+     *     public void onNotifyCursorPosition(TextBuffer source) {
+     *          final int offset;
+     *          
+     *          offset = buffer.getCursorPosition();
+     *     }
+     * });
+     * </pre>
+     * 
+     * If you've already got the <var>insert</var> TextMark, then you could
+     * instead do:
+     * 
+     * <pre>
+     * buffer.connect(new TextBuffer.NotifyCursorPosition()) {
+     *     public void onNotifyCursorPosition(TextBuffer source) {
+     *          final TextIter pointer;
+     *          final int offset;
+     *          
+     *          pointer = buffer.getIter(insertBound);
+     *          offset = buffer.getOffset();
+     *     }
+     * });
+     * </pre>
+     * 
+     * which amounts to the same thing.
+     * 
+     * <p>
+     * Using the <code>TextBuffer.NotifyCursorPosition</code> signal is much
+     * more efficient than hooking up to <code>TextBuffer.MarkSet</code>
+     * signals.
+     * 
+     * @author Andrew Cowie
+     * @since 4.0.10
+     */
+    public interface NotifyCursorPosition extends GtkTextBufferOverride.NotifyCursorPositionSignal
+    {
+        public void onNotifyCursorPosition(TextBuffer source);
+    }
+
+    /**
+     * Hook up a handler to receive
+     * <code>TextBuffer.NotifyCursorPosition</code> signals emitted when the
+     * <var>cursor-position</var> of this TextBuffer changes.
+     * 
+     * @since 4.0.10
+     */
+    public void connect(TextBuffer.NotifyCursorPosition handler) {
+        GtkTextBufferOverride.connect(this, handler, false);
+    }
+
+    /**
+     * Get the current value of the <var>cursor-position</var> property.
+     * 
+     * <p>
+     * This corresponds to the offset of the <var>insert</var> TextMark; if
+     * you already have a TextIter for that you call it's
+     * {@link TextIter#getOffset() getOffset()} instead.
+     * 
+     * @since 4.0.10
+     */
+    public int getCursorPosition() {
+        return getPropertyInteger("cursor-position");
+    }
+
+    /**
+     * Mark the beginning of a user initiated action. Calls to
+     * <code>beginUserAction()</code> can nest, but they need to be paired
+     * with calls to {@link #endUserAction() endUserAction()}. The outer-most
+     * call to this method will raise <code>TextBuffer.BeginUserAction</code>.
+     * Note that user input into a TextView that is handled by GTK's default
+     * <code>Widget.KeyPressEvent</code> handler will likewise begin a user
+     * action sequence.
+     * 
+     * @since 4.0.11
+     */
+    public void beginUserAction() {
+        GtkTextBuffer.beginUserAction(this);
+    }
+
+    /**
+     * Calls to <code>endUserAction()</code> close off the matching
+     * {@link #beginUserAction() beginUserAction()} call.
+     * <code>TextBuffer.EndUserAction</code> will only be emitted when the
+     * outer most user action is closed.
+     * 
+     * @since 4.0.11
+     */
+    public void endUserAction() {
+        GtkTextBuffer.endUserAction(this);
+    }
+
+    /**
+     * Signal emitted when a "user action" is initiated. This can be fired
+     * programmatically by calling TextBuffer's
+     * {@link TextBuffer#beginUserAction() beginUserAction()}, but is also
+     * raised by the default Input Method handler when the user types into a
+     * TextView.
+     * 
+     * @author Andrew Cowie
+     * @since 4.0.11
+     */
+    public interface BeginUserAction extends GtkTextBuffer.BeginUserActionSignal
+    {
+        public void onBeginUserAction(TextBuffer source);
+    }
+
+    /**
+     * Hookup a <code>TextBuffer.BeginUserAction</code> signal handler.
+     * 
+     * @since 4.0.11
+     */
+    public void connect(BeginUserAction handler) {
+        GtkTextBuffer.connect(this, handler, false);
+    }
+
+    /**
+     * The signal emitted when a "user action" stops. See TextBuffer's
+     * {@link TextBuffer#beginUserAction() beginUserAction()} for details.
+     * 
+     * @author Andrew Cowie
+     * @since 4.0.11
+     */
+    public interface EndUserAction extends GtkTextBuffer.EndUserActionSignal
+    {
+        public void onEndUserAction(TextBuffer source);
+    }
+
+    /**
+     * Hookup a <code>TextBuffer.EndUserAction</code> signal handler.
+     * 
+     * @since 4.0.11
+     */
+    public void connect(EndUserAction handler) {
+        GtkTextBuffer.connect(this, handler, false);
+    }
+
+    /**
+     * Tell a currently active <code>TextBuffer.InsertText</code> signal
+     * emission to stop.
+     * 
+     * <p>
+     * Calling this only makes sense within the scope of a normal handler of
+     * that signal; the effect is to prevent insertion by GTK's default
+     * handler.
+     * 
+     * @since 4.0.13
+     */
+    public void stopInsertText() {
+        GtkTextBufferOverride.stopInsertText(this);
+    }
+
+    /**
+     * Tell a currently active <code>TextBuffer.DeleteRange</code> signal
+     * emission to stop.
+     * 
+     * @since 4.0.13
+     */
+    public void stopDeleteRange() {
+        GtkTextBufferOverride.stopDeleteRange(this);
     }
 }
