@@ -1,12 +1,20 @@
 /*
- * ValidateTreeModel.java
+ * java-gnome, a UI library for writing GTK and GNOME programs from Java!
  *
- * Copyright (c) 2007-2008 Operational Dynamics Consulting Pty Ltd, and Others
- * 
- * The code in this file, and the suite it is a part of, are made available
- * to you by the authors under the terms of the "GNU General Public Licence,
- * version 2" See the LICENCE file for the terms governing usage and
- * redistribution.
+ * Copyright © 2007-2010 Operational Dynamics Consulting, Pty Ltd and Others
+ *
+ * The code in this file, and the program it is a part of, is made available
+ * to you by its authors as open source software: you can redistribute it
+ * and/or modify it under the terms of the GNU General Public License version
+ * 2 ("GPL") as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE. See the GPL for more details.
+ *
+ * You should have received a copy of the GPL along with this program. If not,
+ * see http://www.gnu.org/licenses/. The authors of this program may be
+ * contacted through http://java-gnome.sourceforge.net/.
  */
 package org.gnome.gtk;
 
@@ -18,7 +26,7 @@ import org.gnome.gdk.Pixbuf;
 /**
  * @author Andrew Cowie
  */
-public class ValidateTreeModel extends TestCaseGtk
+public class ValidateTreeModel extends GraphicalTestCase
 {
     public final void testListStoreConstructorArguments() {
         try {
@@ -347,12 +355,12 @@ public class ValidateTreeModel extends TestCaseGtk
 
     public final void testSettingValueReferenceColumn() {
         final ListStore model;
-        final DataColumnReference column;
+        final DataColumnReference<File> column;
         final TreeIter row;
         final File target;
 
         model = new ListStore(new DataColumn[] {
-            column = new DataColumnReference(),
+            column = new DataColumnReference<File>(),
         });
 
         row = model.appendRow();
